@@ -245,11 +245,18 @@ function getSortedKeys(obj) {
   return keys;
 }
 
-// Gets a local datestring from a UNIX timestamp in milliseconds.
-function getLocalTimestring(timestamp_ms) {
-  var when = new Date();
-  when.setTime(timestamp_ms);
-  return when.toLocaleString();
+// Convert milliseconds since the epoch to an ISO8601 datestring.
+// Consider using new Date().toISOString() instead (J.S 1.8+)
+function getIso8601String(timestamp_ms) {
+  var time = new Date();
+  time.setTime(timestamp_ms);
+  return '' +
+      time.getUTCFullYear() + '-' +
+      leftPadNumber(time.getUTCMonth() + 1, 2, '0') + '-' +
+      leftPadNumber(time.getUTCDate(), 2, '0') + 'T' +
+      leftPadNumber(time.getUTCHours(), 2, '0') + ':' +
+      leftPadNumber(time.getUTCMinutes(), 2, '0') + ':' +
+      leftPadNumber(time.getUTCSeconds(), 2, '0') + 'Z';
 }
 
 function leftPadNumber(number, minSize, paddingChar) {
@@ -328,7 +335,7 @@ function initJobOverview(jobs, cursor) {
     var activity = '' + job.active_shards + ' / ' + job.shards + ' shards';
     row.append($('<td>').text(activity))
 
-    row.append($('<td>').text(getLocalTimestring(job.start_timestamp_ms)));
+    row.append($('<td>').text(getIso8601String(job.start_timestamp_ms)));
 
     row.append($('<td>').text(getElapsedTimeString(
         job.start_timestamp_ms, job.updated_timestamp_ms)));
@@ -550,7 +557,7 @@ function refreshJobDetail(jobId, detail) {
   $('<li>')
     .append($('<span class="param-key">').text('Start time'))
     .append($('<span>').text(': '))
-    .append($('<span class="param-value">').text(getLocalTimestring(
+    .append($('<span class="param-value">').text(getIso8601String(
           detail.start_timestamp_ms)))
     .appendTo(jobParams);
 
@@ -585,12 +592,44 @@ function refreshJobDetail(jobId, detail) {
   // Graph image.
   var detailGraph = $('#detail-graph');
   detailGraph.empty();
-  $('<div>').text('Processed items per shard').appendTo(detailGraph);
-  $('<img>')
-    .attr('src', detail.chart_url)
-    .attr('width', detail.chart_width || 300)
-    .attr('height', 200)
-    .appendTo(detailGraph);
+  var chartTitle = 'Processed items per shard';
+  if (detail.chart_data) {
+    var data = new google.visualization.DataTable();
+    data.addColumn('string', 'Shard');
+    data.addColumn('number', 'Count');
+    var shards = detail.chart_data.length;
+    for (var i = 0; i < shards; i++) {
+      data.addRow([i.toString(), detail.chart_data[i]]);
+    }
+    var log2Shards = Math.log(shards) / Math.log(2);
+    var chartWidth = Math.max(Math.max(300, shards * 2), 100 * log2Shards);
+    var chartHeight = 200;
+    var options = {
+        legend: 'none',
+        bar: {
+            groupWidth: '100%'
+        },
+        vAxis: {
+          minValue: 0
+        },
+        title: chartTitle,
+        chartArea: {
+          width: chartWidth,
+          height: chartHeight
+        },
+        width: 80 + chartWidth,
+        height: 80 + chartHeight
+    };
+    var chart = new google.visualization.ColumnChart(detailGraph[0]);
+    chart.draw(data, options);
+  } else {
+    $('<div>').text(chartTitle).appendTo(detailGraph);
+    $('<img>')
+      .attr('src', detail.chart_url)
+      .attr('width', detail.chart_width || 300)
+      .attr('height', 200)
+      .appendTo(detailGraph);
+  }
 
   // Aggregated counters.
   var aggregatedCounters = $('#aggregated-counters');
